!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA GSFC Land Data Toolkit (LDT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
! !MODULE: ECVsm_obsMod
! 
! !DESCRIPTION: 
! This module handles the observation plugin for the 
! soil moisture Essential Climate Variable (ECV) product. 
!
! Soil moisture is recognized as an Essential Climate Variable (ECV) by NASA, 
! European Space Agency (ESA), and other agencies and institutions.  

! The Water Cycle Multi-mission Observation Strategy (WACMOS) and Climate Change 
! Initiative (CCI) Soil Moisture projects have released a 32-year harmonized
! product, from 1978-2010, and referred to as the ECV soil moisture dataset. 
! The product includes both active and passive microwave sensor retrievals.  The 
! active dataset provided by the University of Vienna (TU Wien) uses ERS-1, ERS-2 and
! METOP-A C-band scatterometers.  The passive microwave dataset is developed by
! VU University of Amsterdam and NASA, and includes NASA-based measurements from 
! Nimbus 7 SSMR, DMSP SSM/I, TRMM TMI and Aqua AMSR-E sensors.
!
! The merged dataset uses a fixed ranking system based on error derivations
! from triple collocation.  So if AMSR has an higher error than ASCAT, then ASCAT is used
! (and visa versa, see for example Liu et al.,RSE 2012 and Liu et al., HESS 2011).
! In the near future, a dynamic weighting function is planned to be used within the
! merging routine (considered research in progress).
!
! Further information about the product can be obtained at:
! http://www.esa-soilmoisture-cci.org/
!
! REFERENCES: 
! Liu, Y. Y., W. A. Dorigo, et al. (2012). "Trend-preserving blending of passive and 
! active microwave soil moisture retrievals." Remote Sensing of Environment 123: 280-297.
!
! Wagner, W., Dorigo, W., de Jeu, R., Fernandez, D., Benveniste, J., Haas, E., 
! and Ertl, M.: Fusion of active and passive microwave observations to create 
! an Essential Climate Variable data record on soil moisture. ISPRS Ann. 
! Photogramm. Remote Sens. Spatial Inf. Sci., I-7, 315-321, 
! doi:10.5194/isprsannals-I-7-315-2012, 2012.
!
!   
! !REVISION HISTORY: 
!  01 Oct 2012: Sujay Kumar, Initial Specification
!
module ECVsm_obsMod
! !USES: 
  use ESMF
  use map_utils

  implicit none

  PRIVATE 
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  PUBLIC :: ECVsm_obsinit
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  PUBLIC :: ECVsmobs
!EOP
  type, public :: ecvsmobsdec

     character*100          :: odir
     integer                :: mo
     integer                :: version
     real,    allocatable   :: smobs(:,:)
     logical                :: startmode 
     integer                :: ecvnc, ecvnr
     type(proj_info)        :: ecvproj
     integer, allocatable   :: n11(:)
     integer, allocatable   :: n12(:)
     integer, allocatable   :: n21(:)
     integer, allocatable   :: n22(:)
     real,  allocatable     :: w11(:)
     real,  allocatable     :: w12(:)
     real,  allocatable     :: w21(:)
     real,  allocatable     :: w22(:)
  end type ecvsmobsdec

  type(ecvsmobsdec), allocatable:: ECVsmobs(:)

contains
  
!BOP
! 
! !ROUTINE: ECVsm_obsInit
! \label{ECVsm_obsInit}
! 
! !INTERFACE: 
  subroutine ECVsm_obsinit()
! !USES: 
    use ESMF
    use LDT_coreMod,    only : LDT_rc, LDT_config
    use LDT_DAobsDataMod, only : LDT_DAobsData, LDT_initializeDAobsEntry
    use LDT_timeMgrMod, only : LDT_clock, LDT_calendar
    use LDT_logMod,     only : LDT_verify, LDT_logunit

    implicit none
! !ARGUMENTS: 

! 
! !DESCRIPTION: 
!  This subroutine initializes and sets up the data structures required
!  for reading ECV soil moisture data. 
! 
!EOP
    integer            :: npts
    type(ESMF_TimeInterval) :: alarmInterval
    type(ESMF_Time)         :: alarmTime
    integer                 :: status, rc
    real                    :: gridDesci(20)
    integer                 :: n 

    allocate(ECVsmobs(LDT_rc%nnest))

    call ESMF_ConfigFindLabel(LDT_config, &
         'ECV soil moisture observation directory:', rc=status)
    do n=1,LDT_rc%nnest
       call ESMF_ConfigGetAttribute(LDT_Config, ECVsmobs(n)%odir, &
            rc=status)
       call LDT_verify(status, &
            'ECV soil moisture observation directory: not defined')
    enddo

    call ESMF_ConfigFindLabel(LDT_config, &
         'ECV soil moisture version of data:', rc=status)
    do n=1,LDT_rc%nnest
       call ESMF_ConfigGetAttribute(LDT_Config, ECVsmobs(n)%version, &
            rc=status)
       call LDT_verify(status, &
            'ECV soil moisture version of data: not defined')
    enddo

    do n=1,LDT_rc%nnest
       ECVsmobs(n)%startmode = .true. 

       allocate(ECVsmobs(n)%smobs(LDT_rc%lnc(n),LDT_rc%lnr(n)))

       ECVsmobs(n)%smobs = -9999.0
       
       call LDT_initializeDAobsEntry(LDT_DAobsData(n)%soilmoist_obs, &
            "m3/m3",1,1)
       LDT_DAobsData(n)%soilmoist_obs%selectStats = 1
    
       ECVsmobs(n)%ecvnc = 1440
       ECVsmobs(n)%ecvnr = 720

       call map_set(PROJ_LATLON, -89.875,-179.875,&
            0.0, 0.25,0.25, 0.0,&
            ECVsmobs(n)%ecvnc,ECVsmobs(n)%ecvnr,&
            ECVsmobs(n)%ecvproj)
       
       gridDesci = 0 
       gridDesci(1) = 0 
       gridDesci(2) = 1440
       gridDesci(3) = 720
       gridDesci(4) = -89.875
       gridDesci(5) = -179.875
       gridDesci(6) = 128
       gridDesci(7) = 89.875
       gridDesci(8) = 179.875
       gridDesci(9) = 0.25
       gridDesci(10) = 0.25
       gridDesci(20) = 64
    
       
       allocate(ECVsmobs(n)%n11(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(ECVsmobs(n)%n12(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(ECVsmobs(n)%n21(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(ECVsmobs(n)%n22(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       
       allocate(ECVsmobs(n)%w11(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(ECVsmobs(n)%w12(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(ECVsmobs(n)%w21(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(ECVsmobs(n)%w22(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       
       call bilinear_interp_input(n, gridDesci, &
            ECVsmobs(n)%n11, ECVsmobs(n)%n12, &
            ECVsmobs(n)%n21, ECVsmobs(n)%n22, &
            ECVsmobs(n)%w11, ECVsmobs(n)%w12, &
            ECVsmobs(n)%w21, ECVsmobs(n)%w22)
    enddo
  end subroutine ECVsm_obsinit
     
end module ECVsm_obsMod
