!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA GSFC Land Data Toolkit (LDT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
!BOP
! 
! !ROUTINE: readECVsmObs
! \label{readECVsmObs}
! 
! !REVISION HISTORY: 
!  21 July 2010: Sujay Kumar, Initial Specification
! 
! !INTERFACE: 
subroutine readECVsmObs(n)
! !USES:   
  use ESMF
  use LDT_coreMod,      only : LDT_rc
  use LDT_timeMgrMod,   only : LDT_get_julss
  use LDT_logMod,       only : LDT_logunit, LDT_getNextUnitNumber, &
       LDT_releaseUnitNumber
  use LDT_DAobsDataMod
  use ECVsm_obsMod, only : ECVsmobs
  use map_utils

  implicit none
! !ARGUMENTS: 
  integer, intent(in) :: n
! 
! !DESCRIPTION: 
! 
! This subroutine provides the data reader for the ECV
! soil moisture retrieval product. 
!
!EOP

  real              :: timenow
  logical           :: alarmCheck
  logical           :: file_exists
  integer           :: c,r,i,j
  character*100     :: fname
  real              :: smobs(LDT_rc%lnc(n)*LDT_rc%lnr(n))

!-----------------------------------------------------------------------
! It is assumed that CDF is computed using daily observations. 
!-----------------------------------------------------------------------

  timenow = float(LDT_rc%hr)*3600 + 60*LDT_rc%mn + LDT_rc%ss
  alarmcheck = (mod(timenow, 86400.0).eq.0)

  ECVsmobs(n)%smobs = LDT_rc%udef
  smobs= LDT_rc%udef
        
  if(ECVsmobs(n)%startmode.or.alarmCheck) then 
     
     ECVsmobs(n)%startmode = .false. 

     call create_ECVsm_filename(ECVsmobs(n)%odir, &
          ECVsmobs(n)%version,&
          LDT_rc%yr, LDT_rc%mo, LDT_rc%da, fname)

     inquire(file=trim(fname),exist=file_exists)
     if(file_exists) then

        write(LDT_logunit,*) 'Reading ..',trim(fname)
        call read_ECV_data(n, fname,smobs)
     endif

     do r=1,LDT_rc%lnr(n)
        do c=1,LDT_rc%lnc(n)
           if(smobs(c+(r-1)*LDT_rc%lnc(n)).ne.-9999.0) then 
              ECVsmobs(n)%smobs(c,r) = smobs(c+(r-1)*LDT_rc%lnc(n))
           endif
        enddo
     enddo
  endif

  call LDT_logSingleDAobs(n,LDT_DAobsData(n)%soilmoist_obs,&
       ECVsmobs(n)%smobs,vlevel=1)

end subroutine readECVsmObs


!BOP
! 
! !ROUTINE: read_ECV_data
! \label(read_ECV_data)
!
! !INTERFACE:
subroutine read_ECV_data(n, fname, smobs_ip)
! 
! !USES:   
#if(defined USE_NETCDF3 || defined USE_NETCDF4)
  use netcdf
#endif
  use LDT_coreMod,  only : LDT_rc, LDT_domain
  use LDT_logMod,   only : LDT_verify
  use map_utils,    only : latlon_to_ij
  use ECVsm_obsMod, only : ECVsmobs

  implicit none
!
! !INPUT PARAMETERS: 
! 
  integer                       :: n 
  character (len=*)             :: fname
  real                          :: smobs_ip(LDT_rc%lnc(n)*LDT_rc%lnr(n))


! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
!  This subroutine reads the ECV NETCDF file and applies the data
!  quality flags to filter the data. The retrievals are rejected when 
!  land surface temperature is below freezing, if rain is present, if 
!  RFI is present, if residual error is above 0.5 or if optical depth
!  is above 0.8. Finally the routine combines both the C-band and X-band
!  retrievals. 
! 
!  The arguments are: 
!  \begin{description}
!  \item[n]            index of the nest
!  \item[fname]        name of the ECV AMSR-E file
!  \item[smobs\_ip]    soil moisture data processed to the LDT domain
! \end{description}
!
! !FILES USED:
!
! !REVISION HISTORY: 
! 
!EOP
  integer           :: sm(ECVsmobs(n)%ecvnc,ECVsmobs(n)%ecvnr)
  integer           :: flag(ECVsmobs(n)%ecvnc,ECVsmobs(n)%ecvnr)

  real              :: sm_combined(ECVsmobs(n)%ecvnc,ECVsmobs(n)%ecvnr)
  real              :: sm_data(ECVsmobs(n)%ecvnc*ECVsmobs(n)%ecvnr)
  logical*1         :: sm_data_b(ECVsmobs(n)%ecvnc*ECVsmobs(n)%ecvnr)
  logical*1         :: smobs_b_ip(LDT_rc%lnc(n)*LDT_rc%lnr(n))

  integer           :: c,r,i,j
  real              :: ri,rj
  integer           :: nid
  integer           :: smid, flagid
  integer           :: ios

#if(defined USE_NETCDF3 || defined USE_NETCDF4)
  ios = nf90_open(path=trim(fname),mode=NF90_NOWRITE,ncid=nid)
  call LDT_verify(ios,'Error opening file '//trim(fname))
  
  ios = nf90_inq_varid(nid, 'sm',smid)
  call LDT_verify(ios, 'Error nf90_inq_varid: sm')
  
  ios = nf90_inq_varid(nid, 'flag',flagid)
  call LDT_verify(ios, 'Error nf90_inq_varid: flag')
  
  !values
  ios = nf90_get_var(nid, smid, sm)
  call LDT_verify(ios, 'Error nf90_get_var: sm')
  
  ios = nf90_get_var(nid, flagid,flag)
  call LDT_verify(ios, 'Error nf90_get_var: flag')
  
  ios = nf90_close(ncid=nid)
  call LDT_verify(ios,'Error closing file '//trim(fname))

  do r=1, ECVsmobs(n)%ecvnr
     do c=1, ECVsmobs(n)%ecvnc
!------------------------------------------------------------------------
! All data flagged for snow coverage or temperature below zero (flag=1), 
! dense vegetation (flag=2) and no convergence in the ECV algorithm 
! (flag =3) and undefined values are masked out. 
!------------------------------------------------------------------------
        
        if(flag(c,r).ne.0.or.sm(c,r).lt.0) then 
           sm_combined(c,ECVsmobs(n)%ecvnr-r+1) = LDT_rc%udef
        else
           sm_combined(c,ECVsmobs(n)%ecvnr-r+1) = sm(c,r)*0.0001
        endif
     enddo
  enddo
 
  do r=1, ECVsmobs(n)%ecvnr
     do c=1, ECVsmobs(n)%ecvnc
        sm_data(c+(r-1)*ECVsmobs(n)%ecvnc) = sm_combined(c,r)
        if(sm_combined(c,r).ne.LDT_rc%udef) then 
           sm_data_b(c+(r-1)*ECVsmobs(n)%ecvnc) = .true. 
        else
           sm_data_b(c+(r-1)*ECVsmobs(n)%ecvnc) = .false.
        endif
        if(sm_combined(c,r).gt.0.5) then 
           sm_combined(c,r) = LDT_rc%udef
           sm_data_b(c+(r-1)*ECVsmobs(n)%ecvnc) = .false.
        endif
     enddo
  enddo
  
!--------------------------------------------------------------------------
! Interpolate to the LDT running domain
!-------------------------------------------------------------------------- 
  call bilinear_interp(LDT_rc%gridDesc(n,:),&
       sm_data_b, sm_data, smobs_b_ip, smobs_ip, &
       ECVsmobs(n)%ecvnc*ECVsmobs(n)%ecvnr, &
       LDT_rc%lnc(n)*LDT_rc%lnr(n), &
       LDT_domain(n)%lat, LDT_domain(n)%lon,&
       ECVsmobs(n)%w11, ECVsmobs(n)%w12, &
       ECVsmobs(n)%w21, ECVsmobs(n)%w22, &
       ECVsmobs(n)%n11, ECVsmobs(n)%n12, &
       ECVsmobs(n)%n21, ECVsmobs(n)%n22, &
       LDT_rc%udef, ios)

#endif
  
end subroutine read_ECV_data

!BOP
! !ROUTINE: create_ECVsm_filename
! \label{create_ECVsm_filename}
! 
! !INTERFACE: 
subroutine create_ECVsm_filename(ndir, version, yr, mo,da, filename)
! !USES:   

  implicit none
! !ARGUMENTS: 
  character(len=*)  :: filename
  integer           :: version
  integer           :: yr, mo, da
  character (len=*) :: ndir
! 
! !DESCRIPTION: 
!  This subroutine creates the ECV filename based on the time and date 
! 
!  The arguments are: 
!  \begin{description}
!  \item[ndir] name of the ECV soil moisture directory
!  \item[yr]  current year
!  \item[mo]  current month
!  \item[da]  current day
!  \item[filename] Generated ECV filename
! \end{description}
!EOP

  character (len=4) :: fyr
  character (len=2) :: fmo,fda
  
  write(unit=fyr, fmt='(i4.4)') yr
  write(unit=fmo, fmt='(i2.2)') mo
  write(unit=fda, fmt='(i2.2)') da
 
  if(version.eq.1) then 
     filename = trim(ndir)//'/'//trim(fyr)//'/ESACCI-L3S_SOILMOISTURE-SSMV-MERGED-' &
          //trim(fyr)//trim(fmo)//trim(fda)//'000000-fv00.1.nc'
  elseif(version.eq.2) then 
     filename = trim(ndir)//'/'//trim(fyr)//'/ESACCI-SOILMOISTURE-L3S-SSMV-COMBINED-' &
          //trim(fyr)//trim(fmo)//trim(fda)//'000000-fv02.0.nc'
  endif
end subroutine create_ECVsm_filename
