!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v7.2
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
module chirps2_forcingMod
!BOP
! !MODULE: chirps2_forcingMod
! 
! !DESCRIPTION: 
!  This module contains variables and data structures that are used
!  for the implementation of the precipitation product derived from 
! 
!  The implementation in LDT has the derived data type {\tt chirps2\_struc}
!  that includes the variables to specify the runtime options, and 
!  the weights and neighbor information for spatial interpolation
! 
!  They are desribed below: 
! \begin{description}
!  \item[nc]
!    Number of columns (along the east west dimension) for the input data
!  \item[nr]
!    Number of rows (along the north south dimension) for the input data
!  \item[directory]
!    Directory containing the input data
!  \item[griduptime1]
!    The time to switch the input resolution or file naming convention
!  \item[mi]
!    Number of points in the input grid
!  \item[n112,n122,n212,n222]
!    Arrays containing the neighbor information of the input grid
!    for each grid point in LDT, for conservative interpolation.
!  \item[w112,w122,w212,w222]
!    Arrays containing the weights of the input grid
!    for each grid point in LDT, for conservative interpolation.
!  \item[n113]
!    Array containing the weights of the input grid
!    for each grid point in LDT, for neighbor search.
!  \end{description}
!
! !REVISION HISTORY:
!  Yudong Tian,  10/26/2010: Updated to support reading raw .gz files 
!  KR Arsenault, 07/06/2015: Updated to support latest V7 files

! !USES:
  use ESMF

  implicit none
  PRIVATE
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  public :: init_chirps2      !defines the native resolution of
                                 !the input data
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  public :: chirps2_struc

!EOP

  type, public ::  chirps2_type_dec
     real               :: xres, yres
     real               :: ts
     integer            :: nc
     integer            :: nr
     integer            :: mi
     character*80       :: directory  

     real*8             :: chirpstime1, chirpstime2

   ! "Interp" or spatial transform arrays:
   ! Budget-bilinear:
     integer, allocatable  :: n112(:,:)
     integer, allocatable  :: n122(:,:)
     integer, allocatable  :: n212(:,:)
     integer, allocatable  :: n222(:,:)
     real,    allocatable  :: w112(:,:),w122(:,:)
     real,    allocatable  :: w212(:,:),w222(:,:)
   ! Neighbor:
     integer, allocatable  :: n113(:)

  end type chirps2_type_dec
  
  type(chirps2_type_dec), allocatable :: chirps2_struc(:)

contains
  
!BOP
!
! !ROUTINE: init_chirps2
! \label{init_chirps2}
! 
!
! !REVISION HISTORY: 
! 11Dec2003: Sujay Kumar; Initial Specification
! 
! !INTERFACE:
  subroutine init_chirps2(findex)
! !USES: 
    use LDT_coreMod, only : LDT_rc, LDT_config, LDT_domain
    use LDT_timeMgrMod, only : LDT_date2time, LDT_update_timestep, &
                               LDT_tick, LDT_parseTimeString, &
                               LDT_registerAlarm 
    use LDT_logMod,  only : LDT_logunit, LDT_endrun, &
                            LDT_getNextUnitNumber, &
                            LDT_releaseUnitNumber, LDT_verify 
    use ESMF

    implicit none

    integer, intent(in) :: findex
! 
! !DESCRIPTION: 
!  Defines the native resolution of the input forcing for CHIRPS 2.0
!  data. The grid description arrays are based on the netcdf files
!  (see Section~\ref{interp}).
!
!  The routines invoked are: 
!  \begin{description}
!   \item[readconfig\_chirps2](\ref{readconfig_chirps2}) \newline
!     reads the runtime options specified for CHIRPS 2.0 data
!   \item[LDT\_date2time](\ref{LDT_date2time}) \newline
!     converts date to the real time format
!   \item[conserv\_interp\_input](\ref{conserv_interp_input}) \newline
!    computes the neighbor, weights for conservative interpolation
!   \item[neighbor\_interp\_input](\ref{conserv_interp_input}) \newline
!    computes the neighbor, weights for conservative interpolation
!  \end{description}
!
!EOP
    integer :: n 
    integer :: rc             

    integer :: updoy,yr1,mo1,da1,hr1,mn1,ss1
    real    :: upgmt
    integer :: ts1,ts2 

    integer :: LDT_syr,LDT_smo,LDT_sda,LDT_shr,LDT_smn,LDT_sss 
    real*8  :: FirstDateTime
    real*8  :: LDT_StartTime  
    character (len=10):: time 

    real :: gridDesci(20)

! _________________________________________________________________

    allocate(chirps2_struc(LDT_rc%nnest))

    write(LDT_logunit,fmt=*)"MSG: Initializing CHIRPS v2.0 forcing parameters ... "

  ! Read CHIRPS 2.0 config file entries:
    call readconfig_chirps2()


 !- Select and define interp input arrays:
    if( chirps2_struc(1)%xres == 0.05 ) then
      chirps2_struc(:)%nc = 7200
      chirps2_struc(:)%nr = 2000
    elseif( chirps2_struc(1)%xres == 0.25 ) then
      chirps2_struc(:)%nc = 1440
      chirps2_struc(:)%nr = 400
    else
       write(LDT_logunit,*) "MSG: Other resolutions are not available with CHIRPS 2.0" 
       write(LDT_logunit,*) "     precipitation reader, at this time.  Only"
       write(LDT_logunit,*) "     0.05 or 0.25 deg are available ..."
       write(LDT_logunit,*) "Program stopping ... "
       call LDT_endrun
    endif

    LDT_rc%met_nc(findex) = chirps2_struc(1)%nc
    LDT_rc%met_nr(findex) = chirps2_struc(1)%nr

  ! Grid description array:
    gridDesci(:) = 0
    gridDesci(1) = 0
    gridDesci(2) = chirps2_struc(1)%nc
    gridDesci(3) = chirps2_struc(1)%nr
    gridDesci(4) = -50.0 +  ( chirps2_struc(1)%yres/2 )
    gridDesci(5) = -180.0 + ( chirps2_struc(1)%xres/2 )
    gridDesci(6) = 128
    gridDesci(7) = 50.0 -  ( chirps2_struc(1)%yres/2 )
    gridDesci(8) = 180.0 + ( chirps2_struc(1)%xres/2 )
    gridDesci(9)  = chirps2_struc(1)%xres
    gridDesci(10) = chirps2_struc(1)%yres
    gridDesci(20) = 64

    LDT_rc%met_gridDesc(findex,:) = gridDesci(:)

!    print *, LDT_rc%met_gridDesc(findex,1:10)

  ! First date available for the CHIRPS v2.0 dataset:
    yr1 = 1981
    mo1 = 01
    da1 = 01
    hr1 = 0
    mn1 = 0; ss1 = 0
    ts1 = 3600 * 24  ! Need to figure out when CHIRPS precip is valid for ...
    call LDT_tick(FirstDateTime,updoy,upgmt,yr1,mo1,&
         da1,hr1,mn1,ss1,real(ts1) )

  ! Starting date/time for user-specified inputs:
    LDT_syr = LDT_rc%syr
    LDT_smo = LDT_rc%smo
    LDT_sda = LDT_rc%sda
    LDT_shr = LDT_rc%shr
    LDT_smn = LDT_rc%smn
    LDT_sss = LDT_rc%sss
    ts2 = 0
    call LDT_tick(LDT_StartTime,updoy,upgmt,LDT_syr,LDT_smo,&
         LDT_sda,LDT_shr,LDT_smn,LDT_sss,real(ts2) )

  ! Starting date/time for user-specified inputs:
    if( FirstDateTime .gt. LDT_StartTime ) then
       write(LDT_logunit,*) "LDT start time is earlier than CHIRPS 2.0 data availability time!"
       write(LDT_logunit,*) "Program stopping ... "
       call LDT_endrun
    endif

    LDT_rc%met_nf(findex) = 2          ! number of met variables in CHIRPS forcing
    LDT_rc%met_ts(findex) = 24*3600    ! seconds per day
    LDT_rc%met_zterp(findex) = .false.
    LDT_rc%met_proj(findex)  = "latlon"


 !- If only processing parameters, then return to main routine calls ...
    if( LDT_rc%runmode == "LSM parameter processing" ) return
       
    do  n=1,LDT_rc%nnest

       chirps2_struc(n)%mi = chirps2_struc(n)%nc*chirps2_struc(n)%nr

       chirps2_struc(n)%ts = 24 * 3600.
       call LDT_update_timestep(LDT_rc, n, chirps2_struc(n)%ts)

       call LDT_registerAlarm("CHIRPS 2.0 alarm",&
            chirps2_struc(n)%ts,&
            chirps2_struc(n)%ts )

     ! Set local - 1 hour timestep (to replicate model timestep):
       call LDT_update_timestep(LDT_rc, n, 3600.) 

    !- Select and define interp input arrays:
       select case( LDT_rc%met_gridtransform(findex) )

        case( "budget-bilinear" )

          allocate(chirps2_struc(n)%n112(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%n122(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%n212(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%n222(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%w112(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%w122(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%w212(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(chirps2_struc(n)%w222(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))

          call conserv_interp_input(n,gridDesci(:),&
               chirps2_struc(n)%n112,chirps2_struc(n)%n122,&
               chirps2_struc(n)%n212,chirps2_struc(n)%n222,&
               chirps2_struc(n)%w112,chirps2_struc(n)%w122,&
               chirps2_struc(n)%w212,chirps2_struc(n)%w222)

        case( "neighbor" )   

          allocate(chirps2_struc(n)%n113(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          call neighbor_interp_input(n,gridDesci(:),&
               chirps2_struc(n)%n113)

        case default
          write(LDT_logunit,*) "This interpolation option not defined yet for CHIRPS data"
          write(LDT_logunit,*) "Program stopping ... "
          call LDT_endrun

       end select

    enddo

  end subroutine init_chirps2

end module chirps2_forcingMod
