!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v7.1 
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
! !ROUTINE: timeinterp_chirps2
! \label{timeinterp_chirps2}
!
! !REVISION HISTORY:
!
! 16 July 2015: K. Arsenault Hall; Initial LDT version
!
! !INTERFACE:
subroutine timeinterp_chirps2(n,findex)
! !USES:
  use ESMF
  use LDT_coreMod,      only : LDT_rc,LDT_domain
  use LDT_metforcingMod,only :  LDT_FORC_Base_State, LDT_forc
  use LDT_FORC_AttributesMod
  use LDT_logMod,       only : LDT_verify
  use chirps2_forcingMod, only : chirps2_struc

  implicit none

! !ARGUMENTS:
  integer, intent(in) :: n
  integer, intent(in) :: findex

!
! !DESCRIPTION:
!  Temporally interpolates the forcing data to the current model
!  timestep (evenly distributed over sub-daily timesteps for now). 
!
!  The arguments are:
!  \begin{description}
!  \item[n]
!    index of the nest
!  \item[findex]
!    index of the supplemental forcing source
!  \item[metdata2]
!    current forcing values
!  \end{description}
!
!EOP

    integer            :: t, index1
    integer            :: status
    type(ESMF_Field)   :: pcpField, cpcpField
    real, pointer      :: pcp(:), cpcp(:)
    real,  allocatable :: ratio(:)
! ________________________________________________________

    call ESMF_StateGet(LDT_FORC_Base_State(n,findex),trim(LDT_FORC_Rainf%varname(1)),pcpField,&
         rc=status)
    call LDT_verify(status, 'Error: enable Rainf in forcing variables list')

    call ESMF_FieldGet(pcpField,localDE=0,farrayPtr=pcp,rc=status)
    call LDT_verify(status)

!-- Convert precipitation sum to rate:
    if( LDT_FORC_CRainf%selectOpt .ne. 1 ) then
 
      do t=1,LDT_rc%ntiles(n)
         index1 = LDT_domain(n)%tile(t)%index
       ! Leave out undefined and NODATA value grid cells if after upscaling:
         if( LDT_forc(n,findex)%metdata2(1,index1) .ge. 0.0 ) then
           pcp(t) = LDT_forc(n,findex)%metdata2(1,index1) / 86400.0  ! mm/s for entire day
         endif
      enddo

!-- Applying a convective rainfall amount to observed precip field: 
    elseif( LDT_FORC_CRainf%selectOpt .eq. 1 ) then
 
      allocate(ratio(LDT_rc%ntiles(n)))

      call ESMF_StateGet(LDT_FORC_Base_State(n,findex),trim(LDT_FORC_CRainf%varname(1)),cpcpField,&
           rc=status)
      call LDT_verify(status, 'Error: enable CRainf in forcing variables list')

      call ESMF_FieldGet(cpcpField,localDE=0,farrayPtr=cpcp,rc=status)
      call LDT_verify(status)

!------------------------------------------------------------------------
! Compute ratio between convective model precip and total model precip
! so that it can be applied to the observed global precip
!------------------------------------------------------------------------
      do t = 1,LDT_rc%ntiles(n)
         if (pcp(t) .ne. 0.0 .and.  &
             pcp(t) .ne. LDT_rc%udef .and.  &
             cpcp(t) .ne. LDT_rc%udef) then
           ratio(t) = cpcp(t) / pcp(t)
           if (ratio(t) .gt. 1.0) ratio(t) = 1.0
           if (ratio(t) .lt. 0.0) ratio(t) = 0.0
         else
           ratio(t) = 0.0
         endif
      enddo
      do t=1,LDT_rc%ntiles(n)
         index1 = LDT_domain(n)%tile(t)%index
       ! Leave out undefined and NODATA value grid cells if after upscaling:
         if( LDT_forc(n,findex)%metdata2(1,index1) .ge. 0.0 ) then    
           pcp(t) = LDT_forc(n,findex)%metdata2(1,index1) / 86400.0 ! KRA: mm/s for entire day
           cpcp(t) = ratio(t) * pcp(t)
         endif
      enddo
      deallocate(ratio)
    endif

end subroutine timeinterp_chirps2
