!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
module geos_forcingMod
!BOP
! !MODULE: geos_forcingMod
! 
! !DESCRIPTION: 
!  This module contains variables and data structures that are used
!  for the implementation of the GEOS forcing data. 
!  The data is global 1 degree dataset in latlon
!  projection, and at 3 hourly intervals. The derived
!  data type {\tt geos\_struc}
!  includes the variables that specify the runtime options, and the 
!  weights and neighbor information to be used for spatial interpolation. 
!  They are described below: 
!  \begin{description}
!  \item[nc]
!    Number of columns (along the east west dimension) for the input data
!  \item[nr]
!    Number of rows (along the north south dimension) for the input data
!  \item[nmif]
!    Number of forcing variables in the ECMWF data
!  \item[geostime1]
!    The nearest, previous 3 hour instance of the incoming 
!    data (as a real time). 
!  \item[geostime2]
!    The nearest, next 3 hour instance of the incoming 
!    data (as a real time).
!  \item[griduptime]
!    The time to switch GEOS resolution 
!  \item[geosdir]
!    Directory containing the input data
!  \item[mi]
!    Number of points in the input grid
!  \item[n111,n121,n211,n221]
!    Arrays containing the neighbor information of the input grid 
!    for each grid point in LDT, for bilinear interpolation. 
!  \item[w111,w121,w211,w221]
!    Arrays containing the weights of the input grid 
!    for each grid point in LDT, for bilinear interpolation.
!  \item[n122,n122,n212,n222]
!    Arrays containing the neighbor information of the input grid 
!    for each grid point in LDT, for conservative interpolation. 
!  \item[w112,w122,w212,w222]
!    Arrays containing the weights of the input grid 
!    for each grid point in LDT, for conservative interpolation.
!  \item[findtime1, findtime2]
!   boolean flags to indicate which time is to be read for 
!   temporal interpolation.
!  \end{description}
!
! !USES: 
  implicit none

  PRIVATE
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  public :: init_geos      !defines the native resolution of 
                                  !the input data
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  public :: geos_struc

!EOP
  type, public ::  geos_type_dec
     real    :: ts
     integer :: nc, nr   
     integer :: mi
     integer :: nmif
     character*100 :: geosdir   !GEOS Forcing Directory
     real*8  :: geostime1,geostime2
     real*8  :: griduptime1, griduptime2
     logical :: gridchange1, gridchange2
  !Suffixes 1 are for bilinear 
     integer, allocatable   :: n111(:)
     integer, allocatable   :: n121(:)
     integer, allocatable   :: n211(:)
     integer, allocatable   :: n221(:)
     real, allocatable      :: w111(:),w121(:)
     real, allocatable      :: w211(:),w221(:)
     
  !Suffixes 2 are for conservative 
     integer, allocatable   :: n112(:,:)
     integer, allocatable   :: n122(:,:)
     integer, allocatable   :: n212(:,:)
     integer, allocatable   :: n222(:,:)
     real, allocatable      :: w112(:,:),w122(:,:)
     real, allocatable      :: w212(:,:),w222(:,:)
     integer            :: findtime1, findtime2
  end type geos_type_dec
  
  type(geos_type_dec), allocatable :: geos_struc(:)

contains
  
!BOP
!
! !ROUTINE: init_geos
! \label{init_geos}
!
! !REVISION HISTORY: 
! 11Dec2003: Sujay Kumar; Initial Specification
! 
! !INTERFACE:
  subroutine init_geos(findex)
! !USES: 
    use LDT_coreMod,    only :  LDT_rc
    use LDT_timeMgrMod, only : LDT_date2time, LDT_update_timestep
    use LDT_logMod,     only : LDT_logunit,LDT_endrun

    implicit none
! !ARGUMENTS: 
    integer,  intent(in)    :: findex
! 
! !DESCRIPTION: 
!  Defines the native resolution of the input forcing for GEOS
!  data. The grid description arrays are based on the decoding
!  schemes used by NCEP and followed in the LDT interpolation
!  schemes \ref{interp}
!
!  The routines invoked are: 
!  \begin{description}
!   \item[readcrd\_geos](\ref{readcrd_geos}) \newline
!     reads the runtime options specified for GEOS data
!   \item[bilinear\_interp\_input](\ref{bilinear_interp_input}) \newline
!    computes the neighbor, weights for bilinear interpolation
!   \item[conserv\_interp\_input](\ref{conserv_interp_input}) \newline
!    computes the neighbor, weights for conservative interpolation
!  \end{description}
!EOP
    integer :: n
    integer :: updoy, yr1,mo1,da1,hr1,mn1,ss1
    real    :: upgmt
    real    :: gridDesci(20)

    allocate(geos_struc(LDT_rc%nnest))

    write(LDT_logunit,fmt=*)"MSG: Initializing GEOS forcing grid ... "

!    call readcrd_geos()

    LDT_rc%met_nf(findex) = 9
    LDT_rc%met_ts(findex) = 3*3600
    LDT_rc%met_zterp(findex) = .true.

    geos_struc%nc = 360
    geos_struc%nr = 181
    LDT_rc%met_nc(findex) = geos_struc(1)%nc
    LDT_rc%met_nr(findex) = geos_struc(1)%nr

 !- GEOS Grid description:
    LDT_rc%met_proj(findex)        = "latlon"
    LDT_rc%met_gridDesc(findex,1)  = 0
    LDT_rc%met_gridDesc(findex,2)  = LDT_rc%met_nc(findex)
    LDT_rc%met_gridDesc(findex,3)  = LDT_rc%met_nr(findex)
    LDT_rc%met_gridDesc(findex,4)  = -90.000
    LDT_rc%met_gridDesc(findex,5)  = -180.000
    LDT_rc%met_gridDesc(findex,6)  = 128
    LDT_rc%met_gridDesc(findex,7)  = 90.000
    LDT_rc%met_gridDesc(findex,8)  = 179.000
    LDT_rc%met_gridDesc(findex,9)  = 1.000
    LDT_rc%met_gridDesc(findex,10) = 1.000
    LDT_rc%met_gridDesc(findex,20) = 0

    gridDesci(:) = LDT_rc%met_gridDesc(findex,:)

    geos_struc%mi = geos_struc%nc*geos_struc%nr

 !- If only processing parameters, then return to main routine calls ...
    if( LDT_rc%runmode == "LSM parameter processing" ) return

#if 0
    do n=1,LDT_rc%nnest

       geos_struc(n)%ts = 3*3600
       call LDT_update_timestep(LDT_rc, n, geos_struc(n)%ts)

       yr1 = 2002  !grid update time
       mo1 = 10
       da1 = 01
       hr1 = 0; mn1 = 0; ss1 = 0
       call LDT_date2time(geos_struc(n)%griduptime1,updoy,upgmt,yr1,mo1,da1,hr1,mn1,ss1)

       yr1 = 2007  !grid update time
       mo1 = 3
       da1 = 2
       hr1 = 0; mn1 = 0; ss1 = 0
       call LDT_date2time(geos_struc(n)%griduptime2,updoy,upgmt,yr1,mo1,da1,hr1,mn1,ss1)

       geos_struc(n)%gridchange1 = .true.
       geos_struc(n)%gridchange2 = .true. 

     ! Setting up weights for Interpolation
       if(trim(LDT_rc%met_interp(findex)).eq."bilinear") then 
          allocate(geos_struc(n)%n111(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%n121(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%n211(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%n221(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w111(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w121(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w211(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w221(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          call bilinear_interp_input(n, gridDesci(n,:), &
               geos_struc(n)%n111,geos_struc(n)%n121,&
               geos_struc(n)%n211,geos_struc(n)%n221,&
               geos_struc(n)%w111,geos_struc(n)%w121,&
               geos_struc(n)%w211,geos_struc(n)%w221)

       elseif(trim(LDT_rc%met_interp(findex)).eq."budget-bilinear") then 
          allocate(geos_struc(n)%n111(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%n121(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%n211(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%n221(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w111(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w121(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w211(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          allocate(geos_struc(n)%w221(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
          call bilinear_interp_input(n, gridDesci(n,:), &
               geos_struc(n)%n111,geos_struc(n)%n121,&
               geos_struc(n)%n211,geos_struc(n)%n221,&
               geos_struc(n)%w111,geos_struc(n)%w121,&
               geos_struc(n)%w211,geos_struc(n)%w221)
          allocate(geos_struc(n)%n112(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%n122(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%n212(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%n222(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%w112(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%w122(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%w212(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          allocate(geos_struc(n)%w222(LDT_rc%lnc(n)*LDT_rc%lnr(n),25))
          call conserv_interp_input(n, gridDesci(n,:),,&
               geos_struc(n)%n112,geos_struc(n)%n122,&
               geos_struc(n)%n212,geos_struc(n)%n222,&
               geos_struc(n)%w112,geos_struc(n)%w122,&
               geos_struc(n)%w212,geos_struc(n)%w222)
       endif
    enddo
#endif

  end subroutine init_geos

end module geos_forcingMod
