!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v7.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !ROUTINE: get_merraland
! \label{get_merraland}
!  
!
! !REVISION HISTORY:
!  12 Oct 2009: Eric Kemp, initial version
!  27 May 2010: David Mocko, changed to hourly forcing for LDT6.0
!  25 Oct 2010: David Mocko, updated for LDT6.1 public release
!   5 Apr 2013: Sujay Kumar, updated for the direct use of files from GES-DISC
!  31 May 2013: David Mocko, changes to match MERRA-Land interpolation strategy
!  22 Jan 2015: KR Arsenault, added to LDT
!
! !INTERFACE:
subroutine get_merraland(n,findex)

! !USES:
  use LDT_coreMod
  use LDT_timeMgrMod
  use LDT_logMod
  use LDT_metforcingMod
  use merraland_forcingMod

  implicit none

! !ARGUMENTS:
  integer, intent(in) :: n 
  integer, intent(in) :: findex
!  
! !DESCRIPTION:
!  Opens, reads, and interpolates 1-hourly MERRA-Land forcing.
!  At the beginning of a simulation, the code reads the most
!  recent past data (nearest 1 hour interval), and the nearest
!  future data.  These two datasets are used to temporally
!  interpolate the data to the current model timestep.
!  The strategy for missing data is to go backwards up
!  to 10 days to get forcing at the same time of day.
!
!  The arguments are: 
!  \begin{description}
!  \item[n]
!    index of the nest
!  \end{description}
!
!  The routines invoked are: 
!  \begin{description}
!  \item[LDT\_tick](\ref{LDT_tick}) \newline
!    call to advance or retract time
!  \item[merralandfiles](\ref{merralandfiles}) \newline
!    Puts together appropriate file name for 1 hour intervals
!  \item[read\_merraland](\ref{read_merraland}) \newline
!    call to read the MERRA-Land data and perform spatial interpolation 
!  \end{description}
!EOP
  logical            :: alarmCheck
  integer            :: movetime
  integer            :: order
  integer            :: ferror
  character*140      :: slvname, flxname, radname, mldname, lndname
  integer            :: c,r
  integer            :: yr1, mo1, da1, hr1, mn1, ss1, doy1
  integer            :: yr2, mo2, da2, hr2, mn2, ss2, doy2
  real*8             :: time1,time2, timenow
  real               :: gmt1,gmt2,ts1,ts2
! ______________________________________________________________________

! Read data at/after 00:30z every day and keep in memory. 
  if( LDT_rc%hr.eq.23 ) merraland_struc(n)%dayFlag = .true.

  if(LDT_rc%nts(n).lt.3600) then 
     if ((LDT_rc%hr.eq.0).and.(LDT_rc%mn.ge.30).and.merraland_struc(n)%dayFlag) then
        alarmCheck = .true.
        merraland_struc(n)%dayFlag = .false.
     else
        alarmCheck = .false.
     endif
  elseif(LDT_rc%nts(n).eq.3600) then 
     if ((LDT_rc%hr.eq.0).and.merraland_struc(n)%dayFlag) then
        alarmCheck = .true.
        merraland_struc(n)%dayFlag = .false.
     else
        alarmCheck = .false.
     endif
  else
     write(LDT_logunit,*) 'When running LDT with merra-land, the clock '
     write(LDT_logunit,*) 'should not be running with less than an hour of timestep'
     call LDT_endrun()
  endif

  merraland_struc(n)%findtime1 = 0 
  merraland_struc(n)%findtime2 = 0 
  movetime = 0

  if (alarmCheck.or.merraland_struc(n)%startFlag) then

     merraland_struc(n)%findtime2 = 1
     movetime = 1

     if (merraland_struc(n)%startFlag) then
        merraland_struc(n)%startFlag = .false.
        merraland_struc(n)%findtime1 = 1
        movetime = 0 
     endif

  endif
     
  if (movetime.eq.1) then

!     merraland_struc(n)%merralandtime1 = merraland_struc(n)%merralandtime2
     merraland_struc(n)%merraforc1 = merraland_struc(n)%merraforc2
     merraland_struc(n)%merraforc2 = LDT_rc%udef

  endif

  if (merraland_struc(n)%findtime1==1) then      
     order = 1

     yr1 = LDT_rc%yr
     mo1 = LDT_rc%mo
     da1 = LDT_rc%da
     hr1 = LDT_rc%hr
     mn1 = 0
     ss1 = 0
     if (LDT_rc%mn.lt.30) then
        ts1 = -(60*60)
     else
        ts1 = 0
     endif
     call LDT_tick(time1,doy1,gmt1,yr1,mo1,da1,hr1,mn1,ss1,ts1)

     call merralandfiles(merraland_struc(n)%merralanddir,yr1,mo1,da1, &
          slvname, flxname, radname, mldname, lndname)

     call read_merraland(n,order,findex,slvname,flxname,&
          radname,mldname,lndname,merraland_struc(n)%merraforc1,ferror)
  endif

  if (merraland_struc(n)%findtime2.eq.1) then
     order = 2

     yr2 = LDT_rc%yr
     mo2 = LDT_rc%mo
     da2 = LDT_rc%da
     hr2 = LDT_rc%hr
     mn2 = 0
     ss2 = 0
     if (LDT_rc%mn.lt.30) then
        ts2 = (23*60*60)
     else
        ts2 = (24*60*60)
     endif
     call LDT_tick(time2,doy2,gmt2,yr2,mo2,da2,hr2,mn2,ss2,ts2)

     call merralandfiles(merraland_struc(n)%merralanddir,yr2,mo2,da2, &
          slvname, flxname, radname, mldname, lndname)
     call read_merraland(n,order,findex,slvname,flxname,&
          radname,mldname,lndname,merraland_struc(n)%merraforc2,ferror)
  endif

!look up data for the current time.      
  yr1=LDT_rc%yr
  mo1=LDT_rc%mo
  da1=LDT_rc%da
  hr1=LDT_rc%hr
  mn1=LDT_rc%mn
  ss1=0
  ts1=0
  call LDT_tick(timenow,doy1,gmt1,yr1,mo1,da1,hr1,mn1,ss1,ts1)

  if (timenow.ge.merraland_struc(n)%merralandtime2) then
     yr1=LDT_rc%yr
     mo1=LDT_rc%mo
     da1=LDT_rc%da
     hr1=LDT_rc%hr
     mn1=30
     ss1=0
     if (LDT_rc%mn.lt.30) then
        ts1 = -(60*60)
     else
        ts1 = 0
     endif
     call LDT_tick(time1,doy1,gmt1,yr1,mo1,da1,hr1,mn1,ss1,ts1)

     yr2=LDT_rc%yr    !next hour
     mo2=LDT_rc%mo
     da2=LDT_rc%da
     hr2=LDT_rc%hr
     mn2=30
     ss2=0
     if (LDT_rc%mn.lt.30) then
        ts2=0
     else
        ts2=60*60
     endif
     call LDT_tick(time2,doy2,gmt2,yr2,mo2,da2,hr2,mn2,ss2,ts2)

     do r=1,LDT_rc%lnr(n)
        do c=1,LDT_rc%lnc(n)
           if (LDT_domain(n)%gindex(c,r).ne.-1) then
              if (LDT_rc%hr.eq.23) then
                 if (LDT_rc%mn.ge.30) then
                    LDT_forc(n,findex)%metdata1(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc1(:,24,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                    LDT_forc(n,findex)%metdata2(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc2(:,1,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                 else
                    LDT_forc(n,findex)%metdata1(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc1(:,23,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                    LDT_forc(n,findex)%metdata1(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc1(:,24,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                 endif
              else
                 if ((LDT_rc%hr.eq.0).and.(LDT_rc%mn.lt.30)) then
                    LDT_forc(n,findex)%metdata1(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc1(:,24,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                    LDT_forc(n,findex)%metdata2(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc2(:,1,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                 else
                    LDT_forc(n,findex)%metdata1(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc1(:,hr1+1,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                    LDT_forc(n,findex)%metdata2(:,LDT_domain(n)%gindex(c,r)) = &
                         merraland_struc(n)%merraforc1(:,hr1+2,&
                         (c+(r-1)*LDT_rc%lnc(n)))
                 endif
              endif
           endif
        enddo
     enddo

     merraland_struc(n)%merralandtime2 = time2
     merraland_struc(n)%merralandtime1 = time1
  endif

end subroutine get_merraland


!BOP
! !ROUTINE: merralandfiles
! \label{merralandfiles}
!
! !INTERFACE:
subroutine merralandfiles(merralanddir,yr,mo,da, &
     slvname,flxname,radname,mldname,lndname)

! !USES:
  use LDT_logMod, only : LDT_endrun
  
  implicit none
! !ARGUMENTS: 
  character(len=*), intent(in)  :: merralanddir
  integer, intent(in)           :: yr,mo,da
  character(len=*), intent(out) :: slvname
  character(len=*), intent(out) :: flxname
  character(len=*), intent(out) :: radname
  character(len=*), intent(out) :: mldname
  character(len=*), intent(out) :: lndname

! !DESCRIPTION:
!   This subroutine puts together MERRA-land file names for 
!   daily netcdf files
! 
!  The arguments are:
!  \begin{description}
!  \item[merralanddir]
!    Name of the MERRA-Land directory
!  \item[yr]
!    year 
!  \item[mo]
!   month
!  \item[da]
!   day of month
!  \item[slvname]
!   name of the timestamped met forcing file
!  \item[flxname]
!   name of the timestamped precip flux file 
!  \item[radname]
!   name of the timestamped radidation file
!  \end{description}
!
!EOP

  character*6  :: ftime1
  character*8  :: ftime2
  character*10 :: prefix
  logical      :: file_exists

  write(unit=ftime1, fmt='(i4.4,i2.2)') yr,mo
  write(unit=ftime2, fmt='(i4.4,i2.2,i2.2)') yr,mo,da

  if(yr.ge.1993.and.yr.lt.2001) then 
     prefix = 'MERRA20'
  elseif(yr.ge.2001) then 
     prefix = 'MERRA30'
  else
     prefix = 'MERRA10'
  endif
  
! Single Level ("slv") input file:
  slvname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'0.prod.assim.tavg1_2d_slv_Nx.'//&
       trim(ftime2)//'.SUB.nc'

  inquire( file=slvname, exist=file_exists )
  if( .NOT. file_exists ) then
    slvname = trim(merralanddir)//'/'//trim(ftime1)//&
        '/'//trim(prefix)//'1.prod.assim.tavg1_2d_slv_Nx.'//&
        trim(ftime2)//'.SUB.nc'
    inquire( file=slvname, exist=file_exists )
    if( .NOT. file_exists ) then
       print *, " ## ERR MSG:  SLV file missing: ",trim(slvname)
       print *, " LIS endrun being called ..."
       call LDT_endrun
     endif
  endif

! Radiation fields ("rad") input file:
  radname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'1.prod.assim.tavg1_2d_rad_Nx.'//&
       trim(ftime2)//'.SUB.nc'

  inquire( file=radname, exist=file_exists )
  if( .NOT. file_exists ) then
    radname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'0.prod.assim.tavg1_2d_rad_Nx.'//&
       trim(ftime2)//'.SUB.nc'
    inquire( file=radname, exist=file_exists )
    if( .NOT. file_exists ) then
       print *, " ## ERR MSG:  RAD file missing: ",trim(radname)
       print *, " LIS endrun being called ..."
       call LDT_endrun
    endif
  endif

! Flux fields ("flx") input file:
  flxname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'0.prod.assim.tavg1_2d_flx_Nx.'//&
       trim(ftime2)//'.SUB.nc'

  inquire( file=flxname, exist=file_exists )
  if( .NOT. file_exists ) then
    flxname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'1.prod.assim.tavg1_2d_flx_Nx.'//&
       trim(ftime2)//'.SUB.nc'
    inquire( file=flxname, exist=file_exists )
    if( .NOT. file_exists ) then
       print *, " ## ERR MSG:  FLX file missing: ",trim(flxname)
       print *, " LIS endrun being called ..."
       call LDT_endrun
    endif
  endif

! MERRA-Land ("mld") fields (obs ppt) input file:
  mldname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'0.prod.simul.tavg1_2d_mld_Nx.'//&
       trim(ftime2)//'.SUB.nc'

  inquire( file=mldname, exist=file_exists )
  if( .NOT. file_exists ) then
    mldname = trim(merralanddir)//'/'//trim(ftime1)//&
         '/'//trim(prefix)//'1.prod.simul.tavg1_2d_mld_Nx.'//&
         trim(ftime2)//'.SUB.nc'
    inquire( file=mldname, exist=file_exists )
    if( .NOT. file_exists ) then
       print *, " ## ERR MSG:  MLD file missing: ",trim(mldname)
       print *, " LIS endrun being called ..."
       call LDT_endrun
    endif
  endif

! MERRA Land-based ("lnd") fields input file:
  lndname = trim(merralanddir)//'/'//trim(ftime1)//&
       '/'//trim(prefix)//'0.prod.assim.tavg1_2d_lnd_Nx.'//&
       trim(ftime2)//'.SUB.nc'

  inquire( file=lndname, exist=file_exists )
  if( .NOT. file_exists ) then
    lndname = trim(merralanddir)//'/'//trim(ftime1)//&
         '/'//trim(prefix)//'1.prod.assim.tavg1_2d_lnd_Nx.'//&
         trim(ftime2)//'.SUB.nc'
    inquire( file=lndname, exist=file_exists )
    if( .NOT. file_exists ) then
       print *, " ## ERR MSG:  LND file missing: ",trim(lndname)
       print *, " LIS endrun being called ..."
       call LDT_endrun
    endif
  endif

end subroutine merralandfiles

